<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Products;
use App\Models\Category;
use App\Models\Attributes;
use App\Models\AttributeValue;
;


class ProductsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
        $category =  Category::all();
        $products = Products::orderBy('created_at','desc')->paginate(20);
        return  view('product.list')->with('products', $products)->with('category', $category);  
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
        $attributes = Attributes::with('getCustomOption')->get();
 
        $category =  Category::all();
        return  view('product.create')->with('category', $category)->with('attributes', $attributes);          
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //


        $validatedData = $request->validate([
            'name' => 'required|max:255', 
            'code' => 'required|max:255', 
            'descriptions' => 'required', 
            'category' => 'required|max:255', 
            'buyprice' => 'required|max:255', 
        ]);  
        
        if($request->hasFile('product-images')){
            $names = [];
            foreach($request->file('product-images') as $image)
            {
                $filename = md5($image->getClientOriginalName() . time()) . "." . $image->getClientOriginalExtension();  
                $destinationPath = './uploads/products/';
                $image->move($destinationPath, $filename);
                array_push($names, $filename);          
        
            }
     }

        $Products =  Products::create([
            'name' =>  $request->name,
            'sku' =>  $request->code,
            'descriptions' => $request->descriptions,
            'images' => json_encode($names),
            'category' => $request->category,
            'subcategory' => $request->subcategory,
            'buyprice' => $request->buyprice,
            'buypricecurrency' => $request->buypricecurrency,
            'sellprice' => $request->sellprice,
            'sellpricecurrency' => $request->sellpricecurrency,
        ]); 

       $prodcutID = $Products->id;
    
        $rew = $request->attribute;    
        //// foreach to add attributed 
        foreach ($rew as $key => $value) {
                AttributeValue::create([
                       'prodcut_attribute_id' => $key,
                       'value' =>  $value, 
                       'product_id' => $prodcutID    
                    ]);
        }
         if($Products){
            return redirect('/products')->with('success', 'products is successfully saved');
         }   



    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $products =  Products::findOrFail($id);
            return response()->json([
                'data' => $products
            ]) ;   
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $product = Products::findOrFail($id);
        $category =  Category::all();
        $attributes = Attributes::with('getCustomOption')->get();
        $AttributeValue = AttributeValue::where('product_id', $id)->get();
        return  view('product.edit')->with('category', $category)->with('product', $product)->with('attributes', $attributes)->with('AttributeValue', $AttributeValue);     
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
        $validatedData = $request->validate([
            'name' => 'required|max:255', 
            'sku' => 'required|max:255', 
            'descriptions' => 'required', 
            'category' => 'required|max:255', 
            'buyprice' => 'required|max:255', 
        ]);

        $names = []; 
        if($request->hasFile('product-images')){
            $names = [];
            foreach($request->file('product-images') as $image)
            {
                $filename = md5($image->getClientOriginalName() . time()) . "." . $image->getClientOriginalExtension();  
                $destinationPath = './uploads/products/';
                $image->move($destinationPath, $filename);
                array_push($names, $filename);          
        
            }
        } //// foreach

         /// AttributeValue    
        $Products =  Products::whereId($id)->update([
            'name' =>  $request->name,
            'sku' =>  $request->sku,
            'descriptions' => $request->descriptions,
            'images' => json_encode($names),
            'category' => $request->category,
            'subcategory' => $request->subcategory,
            'buyprice' => $request->buyprice,
            'buypricecurrency' => $request->buypricecurrency,
            'sellprice' => $request->sellprice,
            'sellpricecurrency' => $request->sellpricecurrency,
        ]); 
        $prodcutID = $id;
        $rew = $request->attribute;    
        //// foreach to add attributed 
    
        foreach ($rew as $key => $value) {
             $attributesValue = AttributeValue::where('prodcut_attribute_id', $key)->first();
                if($attributesValue){
                    $attributesValue->value = $value;
                    $attributesValue->save();
   /*                 AttributeValue::where('prodcut_attribute_id', $key)->update([
                        'prodcut_attribute_id' => $key,
                        'value' =>  $value, 
                        'product_id' => $prodcutID    
                     ]);  */                  
                } else{

                    AttributeValue::create([
                        'prodcut_attribute_id' => $key,
                        'value' =>  $value, 
                        'product_id' => $prodcutID    
                     ]);                 
                }

             }

         if($Products){
            return redirect('/products')->with('success', 'products is successfully saved');
         }   


    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
        $category->delete();
        return redirect('/categories')->with('success', 'Categories deleted successfully');    
    }

    /*
        getLastID
    */
    public function getLastID(){
    
       $product = Products::latest('id')->first();
       return  $product->id + 1;
    }
}
